module sema.Declarations;

import sema.Visitor,
       sema.DType;

import tango.io.Stdout;

import misc.Error;

class Declarations : Visitor!(void)
{
    int[char[]] types;

    override void visitIdentifier(Identifier i)
    {
        auto symbol = i.env.find(i);

        if(symbol is null)
            throw new Error("Undefined identifier: '"~i.get~"'",i.token.location);

    }

    override void visitVarDecl(VarDecl d)
    {
        if(!d.env.findType(d.type))
            throw new Error("Undefined type: '"~d.type.get~"'",d.type.token.location);

        visitExp(d.identifier);
        if (d.init)
            visitExp(d.init);
    }
    override void visitFuncDecl(FuncDecl f)
    {
        visitExp(f.identifier);

        foreach (stmt; f.statements)
            visitStmt(stmt);
    }

    override void visitMemberLookup(MemberLookup m)
    {
        switch(m.target.expType)
        {
            case ExpType.Identifier:
                auto target = cast(Identifier)m.target;
                auto child = m.child;
                auto st = cast(DStruct)(target.env.find(target).type);
                if((child.get in st.members) == null)
                    throw new Error("Undefined member in "~target.get~" with type "~st.name,st.getLoc);
        }
    }


    bool isType(char[] s)
    {
        return (s in types? true : false);
    }

}

